﻿local Engine, Api, Locale, Settings = unpack(Immersive)

-- Extensions
local debug = Engine.Core.Debug

-- Constants
local X_CONTROL_RIGHT_COLUMN = 289
local Y_CONTROL_OFFSET = 20
local MIN_ANIM_SECONDS = 0
local MAX_ANIM_SECONDS = 60
local ANCHORS = { ["TOPLEFT"] = "Top Left", ["TOPRIGHT"] = "Top Right", ["BOTTOMLEFT"] = "Bottom Left", ["BOTTOMRIGHT"] = "Bottom Right" }
local DIRECTIONS = { ["HORIZONTAL"] = "Horizontal", ["VERTICAL"] = "Vertical" }

local Chat = Engine:GetModule("Chat")
local Handlers = {}

-- Registers the Chat settings panel
function Chat:OnRegisterOptions()
	local settings = Settings:Get(self.Name)
	local panel = CreateFrame("Frame", string.format("%s%sOptionsFrame", Engine.AddOnName, self.Name))
	local element = Api:GetElement("ChatFrame")
	
	panel.name = Locale.CHAT_OPTIONS_NAME
	panel.parent = Locale.CONFIG_OPTIONS_NAME

	panel.okay = Chat.OnOkay
	panel.cancel = Chat.OnCancel
	panel.default = Chat.OnChat
	panel.refresh = Chat.OnRefresh

	-- Description
	local title = panel:CreateFontString("Title", "ARTWORK", "GameFontNormalLarge")
	title:SetJustifyH("LEFT")
	title:SetJustifyV("TOP")
	title:SetPoint("TOPLEFT", panel, "TOPLEFT", 16, -16)
	title:SetText(Locale.CHAT_OPTIONS_TITLE)

	local subText = panel:CreateFontString("Title", "ARTWORK", "GameFontHighlightSmall")
	subText:SetJustifyH("LEFT")
	subText:SetJustifyV("TOP")
	subText:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
	subText:SetPoint("RIGHT", panel, "RIGHT", -32, 0)
	subText:SetText(Locale.CHAT_OPTIONS_SUBTEXT)

	-- Controls: Enable
	local enabled = CreateFrame("CheckButton", string.format("%s%sEnableCheckButton", Engine.AddOnName, self.Name), panel, "InterfaceOptionsCheckButtonTemplate")
	enabled:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", -2, -8)
	enabled:SetChecked(settings.Enabled)
	enabled:SetScript("OnClick", function(self)
		settings.Enabled = self:GetChecked()
	end)
	enabled:SetScript("OnEnter", function(self)
		if self.Tooltip then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
			GameTooltip:SetText(self.Tooltip, nil, nil, nil, nil, true);
		end
	end)
	enabled.Text:SetText(Locale.CHAT_OPTIONS_ENABLED_TEXT)
	enabled.Tooltip = Locale.CHAT_OPTIONS_ENABLED_TOOLTIP
	enabled.OnOkay = function(self)
		settings.Enabled = self:GetChecked()
	end
	enabled.OnRefresh = function(self)
		self:SetValue(settings.Enabled)
	end
	self:AddHandler(delay)

	-- Controls: ScrollFrame
	local scrollframe = CreateFrame("ScrollFrame", nil, panel)
	scrollframe:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", 0, -40)
	scrollframe:SetPoint("BOTTOMRIGHT", -10, 10)
	scrollframe:EnableMouseWheel(true)

	-- Controls: ScrollBar 
	local scrollbar = CreateFrame("Slider", nil, scrollframe, "UIPanelScrollBarTemplate")
	scrollbar:SetPoint("TOPRIGHT", panel, "TOPRIGHT", -8, -24)
	scrollbar:SetPoint("BOTTOMRIGHT", panel, "BOTTOMRIGHT", -8, 24)
	scrollbar:SetValueStep(1)
	scrollbar.scrollStep = 40
	scrollbar:SetWidth(16)
	scrollbar:SetScript("OnValueChanged", function(self, value)
		self:GetParent():SetVerticalScroll(value)
	end)

	-- Controls: Element Container
	local container = CreateFrame("Frame", nil, scrollframe)
	container:SetPoint("TOPLEFT", scrollframe, "TOPLEFT")
	container:SetPoint("BOTTOMRIGHT", scrollframe, "BOTTOMRIGHT")

	scrollframe:SetScrollChild(container)
	scrollframe:SetScript("OnMouseWheel", function(self, delta)
		local min, max = scrollbar:GetMinMaxValues()
		local value = math.max(0, math.min(self:GetVerticalScroll() + (delta * scrollbar.scrollStep * -1), max))
		self:SetVerticalScroll(value)
		scrollbar:SetValue(value)
	end)
	
	local settings = Settings:Get(self.Name)
	local y = 0

	-- Controls: Delay Time
	local delay = CreateFrame("Slider", string.format("%s%sDelaySlider", Engine.AddOnName, self.Name), container, "OptionsSliderTemplate")
	delay:SetPoint("TOPLEFT", container, "TOPLEFT", 24, -15)
	delay:SetMinMaxValues(MIN_ANIM_SECONDS, MAX_ANIM_SECONDS)
	delay:SetValue(settings.DelayTime)
	delay:SetScript("OnValueChanged", function(self)
		self.Text:SetText(string.format(Locale.CONFIG_SLIDER_FORMATTED_TEXT, Locale.CONFIG_SLIDER_TYPE_DELAY_TEXT, math.floor(self:GetValue())))
	end)
	delay.Text = _G[delay:GetName().."Text"]
	delay.Text:SetText(string.format(Locale.CONFIG_SLIDER_FORMATTED_TEXT, Locale.CONFIG_SLIDER_TYPE_DELAY_TEXT, math.floor(delay:GetValue())))
	delay.tooltipText = nil
	delay.tooltipRequirement = nil
	delay.OnOkay = function(self)
		settings.DelayTime = math.floor(self:GetValue())
		element.DelayTime = settings.DelayTime
		element:Apply()
	end
	delay.OnRefresh = function(self)
		self:SetValue(settings.DelayTime)
	end
	self:AddHandler(delay)
	self:AddDependency(enabled, delay)

	-- Controls: Fade Time
	local fade = CreateFrame("Slider", string.format("%s%sFadeSlider", Engine.AddOnName, self.Name), container, "OptionsSliderTemplate")
	fade:SetPoint("TOPLEFT", container, "TOPLEFT", 24, -50)
	fade:SetMinMaxValues(MIN_ANIM_SECONDS, MAX_ANIM_SECONDS)
	fade:SetValue(settings.FadeTime)
	fade:SetScript("OnValueChanged", function(self)
		self.Text:SetText(string.format(Locale.CONFIG_SLIDER_FORMATTED_TEXT, Locale.CONFIG_SLIDER_TYPE_FADE_TEXT, math.floor(self:GetValue())))
	end)
	fade.Text = _G[fade:GetName().."Text"]
	fade.Text:SetText(string.format(Locale.CONFIG_SLIDER_FORMATTED_TEXT, Locale.CONFIG_SLIDER_TYPE_FADE_TEXT, math.floor(fade:GetValue())))
	fade.tooltipText = nil
	fade.tooltipRequirement = nil
	fade.OnOkay = function(self)
		settings.FadeTime = math.floor(self:GetValue())
		element.FadeTime = settings.FadeTime
		element:Apply()
	end
	fade.OnRefresh = function(self)
		self:SetValue(settings.FadeTime)
	end
	self:AddHandler(fade)
	self:AddDependency(enabled, fade)

	-- Controls: Mode
	local mode = CreateFrame("Button", string.format("%s%sModeDropDown", Engine.AddOnName, self.Name), container, "UIDropDownMenuTemplate")
	mode:SetPoint("TOPLEFT", container, "TOPLEFT", 4, -100)
	mode:EnableMouse(true)
	mode:SetScript("OnEnter", function(self)
		if self.Tooltip then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
			GameTooltip:SetText(self.Tooltip, nil, nil, nil, nil, true);
		end
	end)
	mode.Label = mode:CreateFontString("Label", "BACKGROUND", "GameFontHighlight")
	mode.Label:SetPoint("BOTTOMLEFT", mode, "TOPLEFT", 16, 3)
	mode.Label:SetText(Locale.CHAT_OPTIONS_MODE_TITLE)
	mode.Tooltip = Locale.CHAT_OPTIONS_MODE_TOOLTIP
	UIDropDownMenu_Initialize(mode, function(self, level)
		for k, v in ipairs(Api:GetAvailableModes(element)) do
			local info = UIDropDownMenu_CreateInfo()
			info.text = v.Title
			info.value = v.Name
			info.func = function(self)
				UIDropDownMenu_SetSelectedValue(mode, self.value)
			end
			UIDropDownMenu_AddButton(info, level)
		end
	end)
	UIDropDownMenu_SetWidth(mode, 150)
	UIDropDownMenu_SetButtonWidth(mode, 174)
	UIDropDownMenu_SetSelectedValue(mode, settings.Mode)
	UIDropDownMenu_SetText(mode, Api.Modes[settings.Mode].Title)
	UIDropDownMenu_JustifyText(mode, "LEFT")
	mode.OnOkay = function(self)
		settings.Mode = UIDropDownMenu_GetSelectedValue(self)
		element.Mode = settings.Mode
	end
	mode.OnRefresh = function(self)
		UIDropDownMenu_SetSelectedValue(self, settings.Mode)
		UIDropDownMenu_SetText(self, Api.Modes[settings.Mode].Title)
	end
	self:AddHandler(mode)
	self:AddDependency(enabled, mode)

	-- Controls: Anchor
	local anchor = CreateFrame("Button", string.format("%s%sAnchorDropDown", Engine.AddOnName, self.Name), container, "UIDropDownMenuTemplate")
	anchor:SetPoint("TOPLEFT", container, "TOPLEFT", 4, -150)
	anchor:EnableMouse(true)
	anchor:SetScript("OnEnter", function(self)
		if self.Tooltip then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
			GameTooltip:SetText(self.Tooltip, nil, nil, nil, nil, true);
		end
	end)
	anchor.Label = anchor:CreateFontString("Label", "BACKGROUND", "GameFontHighlight")
	anchor.Label:SetPoint("BOTTOMLEFT", anchor, "TOPLEFT", 16, 3)
	anchor.Label:SetText(Locale.CHAT_OPTIONS_ANCHOR_TITLE)
	anchor.Tooltip = Locale.CHAT_OPTIONS_ANCHOR_TOOLTIP
	UIDropDownMenu_Initialize(anchor, function(self, level)
		for k, v in pairs(ANCHORS) do
			local info = UIDropDownMenu_CreateInfo()
			info.text = v
			info.value = k
			info.func = function(self)
				UIDropDownMenu_SetSelectedValue(anchor, self.value)
			end
			UIDropDownMenu_AddButton(info, level)
		end
	end)
	UIDropDownMenu_SetWidth(anchor, 150)
	UIDropDownMenu_SetButtonWidth(anchor, 174)
	UIDropDownMenu_SetSelectedValue(anchor, settings.Anchor)
	UIDropDownMenu_SetText(anchor, ANCHORS[settings.Anchor])
	UIDropDownMenu_JustifyText(anchor, "LEFT")
	anchor.OnOkay = function(self)
		settings.Anchor = UIDropDownMenu_GetSelectedValue(self)
	end
	anchor.OnRefresh = function(self)
		UIDropDownMenu_SetSelectedValue(self, settings.Anchor)
		UIDropDownMenu_SetText(self, ANCHORS[settings.Anchor])
	end
	self:AddHandler(anchor)
	self:AddDependency(enabled, anchor)

	-- Controls: Direction
	local direction = CreateFrame("Button", string.format("%s%sDirectionDropDown", Engine.AddOnName, self.Name), container, "UIDropDownMenuTemplate")
	direction:SetPoint("TOPLEFT", container, "TOPLEFT", 4, -200)
	direction:EnableMouse(true)
	direction:SetScript("OnEnter", function(self)
		if self.Tooltip then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
			GameTooltip:SetText(self.Tooltip, nil, nil, nil, nil, true);
		end
	end)
	direction.Label = direction:CreateFontString("Label", "BACKGROUND", "GameFontHighlight")
	direction.Label:SetPoint("BOTTOMLEFT", direction, "TOPLEFT", 16, 3)
	direction.Label:SetText(Locale.CHAT_OPTIONS_DIRECTION_TITLE)
	direction.Tooltip = Locale.CHAT_OPTIONS_DIRECTION_TOOLTIP
	UIDropDownMenu_Initialize(direction, function(self, level)
		for k, v in pairs(DIRECTIONS) do
			local info = UIDropDownMenu_CreateInfo()
			info.text = v
			info.value = k
			info.func = function(self)
				UIDropDownMenu_SetSelectedValue(direction, self.value)
			end
			UIDropDownMenu_AddButton(info, level)
		end
	end)
	UIDropDownMenu_SetWidth(direction, 150)
	UIDropDownMenu_SetButtonWidth(direction, 174)
	UIDropDownMenu_SetSelectedValue(direction, settings.Direction)
	UIDropDownMenu_SetText(direction, DIRECTIONS[settings.Direction])
	UIDropDownMenu_JustifyText(direction, "LEFT")
	direction.OnOkay = function(self)
		settings.Direction = UIDropDownMenu_GetSelectedValue(self)
	end
	direction.OnRefresh = function(self)
		UIDropDownMenu_SetSelectedValue(self, settings.Direction)
		UIDropDownMenu_SetText(self, DIRECTIONS[settings.Direction])
	end
	self:AddHandler(direction)
	self:AddDependency(enabled, direction)

	-- Controls: Is Hotspot Active
	local hotspot = CreateFrame("CheckButton", string.format("%s%sHotspotCheckButton", Engine.AddOnName, self.Name), container, "InterfaceOptionsCheckButtonTemplate")
	hotspot:SetPoint("TOPLEFT", container, "TOPLEFT", 18, -235)
	hotspot:SetChecked(settings.IsHotspot)
	hotspot:SetScript("OnEnter", function(self)
		if self.Tooltip then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
			GameTooltip:SetText(self.Tooltip, nil, nil, nil, nil, true);
		end
	end)
	hotspot.OnOkay = function(self)
		settings.IsHotspot = self:GetChecked()
		element.IsHotspot = settings.IsHotspot
	end
	hotspot.OnRefresh = function(self)
		self:SetChecked(settings.IsHotspot)
	end
	hotspot.Text:SetText(Locale.CHAT_OPTIONS_HOTSPOT_TEXT)
	hotspot.Tooltip = Locale.CHAT_OPTIONS_HOTSPOT_TOOLTIP
	self:AddHandler(hotspot)
	self:AddDependency(enabled, hotspot)
		
	-- Controls: Conditions
	for name, condition in pairs(Api.Conditions) do
		local conditional = CreateFrame("CheckButton", string.format("%s%s%sConditionCheckButton", Engine.AddOnName, self.Name, name), container, "InterfaceOptionsCheckButtonTemplate")
		conditional:SetPoint("TOPLEFT", container, "TOPLEFT", X_CONTROL_RIGHT_COLUMN, -y)
		conditional:SetChecked(settings.Conditions[name] or false)
		conditional.Text:SetText(condition.Title)
		conditional.Tooltip = condition.Tooltip
		conditional:SetScript("OnEnter", function(self)
			if self.Tooltip then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
				GameTooltip:SetText(self.Tooltip, nil, nil, nil, nil, true);
			end
		end)
		conditional.OnOkay = function(self)
			settings.Conditions[name] = self:GetChecked()
		end
		conditional.OnRefresh = function(self)
			self:SetChecked(settings.Conditions[name] or false)
		end
		self:AddHandler(conditional)
		self:AddDependency(enabled, conditional)
			
		y = y + Y_CONTROL_OFFSET
	end

	container:SetSize(128, y)

	scrollbar:SetMinMaxValues(0, container:GetHeight())
	scrollbar:SetValue(0)

	InterfaceOptions_AddCategory(panel)
end

-- This method will run when the player clicks "okay" in the Interface
-- Options.
function Chat:OnOkay()
	-- Apply changes to elements
	for i = 1, #Handlers do
		Handlers[i]:OnOkay()
	end
end

-- This method will run when the player clicks "cancel" in the Interface
-- Options. Use this to revert their changes.
function Chat:OnCancel()
end

-- This method will run when the player clicks "defaults". Use this to
-- revert their changes to your defaults.
function Chat:OnChat()
end

-- This method will run when the Interface Options frame calls its OnShow
-- function and after defaults have been applied via the panel.default
-- method described above. Use this to refresh your panel's UI in case
-- settings were changed without player interaction.
function Chat:OnRefresh()
	for i = 1, #Handlers do
		Handlers[i]:OnRefresh()
	end
end

-- Adds an element option category to the cache.
function Chat:AddHandler(handler)
	table.insert(Handlers, handler)
end

-- Creates a dependency between a enabled and an enable/disable checkbox.
function Chat:AddDependency(parent, child)
	if parent.Children == nil then
		parent.Children = {}
		parent:SetScript("OnClick", function(self)
			for i = 1, #self.Children do
				if self:GetChecked() then
					self.Children[i]:Enable()
				else
					self.Children[i]:Disable()
				end
			end
		end)
	end

	table.insert(parent.Children, child)
	if parent:GetChecked() then
		child:Enable()
	else
		child:Disable()
	end
end

-- Prompt asking the player to reload the UI.
StaticPopupDialogs["IMMERSIVE_RELOADUI_CONFIRM"] = {
	text = Locale.DEFAULT_RELOAD_REQUIRED,
	button1 = Locale.DEFAULT_RELOAD_REQUIRED_YES,
	button2 = Locale.DEFAULT_RELOAD_REQUIRED_NO,
	OnAccept = function()
		ReloadUI()
	end,
	timeout = 0,
	whileDead = true,
	hideOnEscape = true,
	preferredIndex = 3
}